Package: DESeq2
Type: Package
Title: Differential gene expression analysis based on the negative
        binomial distribution
Version: 1.52.0
Authors@R: c(
    person("Michael", "Love", email="michaelisaiahlove@gmail.com", role = c("aut","cre")),
    person("Constantin", "Ahlmann-Eltze", role = "ctb"),
    person("Anqi", "Zhu", role = "ctb"),
    person("Nikolaos", "Ignatiadis", role = "ctb"),
    person("Raphael", "Rossellini", role = "ctb"),
    person("Kwame", "Forbes", role = "ctb"),
    person("Simon", "Anders", role = c("aut","ctb")),
    person("Wolfgang", "Huber", role = c("aut","ctb")),
    person("RADIANT EU FP7", role="fnd"),
    person("NIH NHGRI", role="fnd"),
    person("CZI", role="fnd"))
Maintainer: Michael Love <michaelisaiahlove@gmail.com>
Description: Estimate variance-mean dependence in count data from
        high-throughput sequencing assays and test for differential
        expression based on a model using the negative binomial
        distribution.
License: LGPL (>= 3)
VignetteBuilder: knitr, rmarkdown
Imports: BiocGenerics (>= 0.7.5), Biobase, BiocParallel, matrixStats,
        methods, stats4, locfit, ggplot2 (>= 3.4.0), Rcpp (>= 0.11.0),
        MatrixGenerics
Depends: S4Vectors (>= 0.23.18), IRanges, GenomicRanges,
        SummarizedExperiment (>= 1.1.6)
Suggests: testthat, knitr, rmarkdown, vsn, pheatmap, RColorBrewer,
        apeglm, ashr, tximport, tximeta, tximportData, readr, pbapply,
        airway, glmGamPoi, BiocManager
LinkingTo: Rcpp, RcppArmadillo
URL: https://github.com/thelovelab/DESeq2
biocViews: Sequencing, RNASeq, ChIPSeq, GeneExpression, Transcription,
        Normalization, DifferentialExpression, Bayesian, Regression,
        PrincipalComponent, Clustering, ImmunoOncology
RoxygenNote: 7.3.3
Encoding: UTF-8
Config/pak/sysreqs: zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:37:22 UTC
RemoteUrl: https://github.com/bioc/DESeq2
RemoteRef: RELEASE_3_23
RemoteSha: 16aeab6d6158bd8cbb8e98764c2f33399b1a3fd3
NeedsCompilation: yes
Packaged: 2026-04-29 13:03:30 UTC; root
Author: Michael Love [aut, cre],
  Constantin Ahlmann-Eltze [ctb],
  Anqi Zhu [ctb],
  Nikolaos Ignatiadis [ctb],
  Raphael Rossellini [ctb],
  Kwame Forbes [ctb],
  Simon Anders [aut, ctb],
  Wolfgang Huber [aut, ctb],
  RADIANT EU FP7 [fnd],
  NIH NHGRI [fnd],
  CZI [fnd]
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-29 13:11:43 UTC; unix
