# Generated by roxygen2: do not edit by hand

S3method(coef,DESeqDataSet)
export("dispersionFunction<-")
export("dispersions<-")
export("normalizationFactors<-")
export("priorInfo<-")
export(DESeq)
export(DESeqDataSet)
export(DESeqDataSetFromHTSeqCount)
export(DESeqDataSetFromMatrix)
export(DESeqDataSetFromTximport)
export(DESeqResults)
export(DESeqTransform)
export(collapseReplicates)
export(dispersionFunction)
export(dispersions)
export(estimateBetaPriorVar)
export(estimateDispersionsFit)
export(estimateDispersionsGeneEst)
export(estimateDispersionsMAP)
export(estimateDispersionsPriorVar)
export(estimateMLEForBetaPriorVar)
export(estimateSizeFactorsForMatrix)
export(fpkm)
export(fpm)
export(getVarianceStabilizedData)
export(lfcShrink)
export(makeExampleDESeqDataSet)
export(nbinomLRT)
export(nbinomWaldTest)
export(normTransform)
export(normalizationFactors)
export(normalizeGeneLength)
export(plotCounts)
export(plotSparsity)
export(priorInfo)
export(removeResults)
export(replaceOutliers)
export(replaceOutliersWithTrimmedMean)
export(results)
export(resultsNames)
export(rlog)
export(rlogTransformation)
export(unmix)
export(varianceStabilizingTransformation)
export(vst)
exportClasses(DESeqDataSet)
exportClasses(DESeqResults)
exportClasses(DESeqTransform)
exportMethods("counts<-")
exportMethods("design<-")
exportMethods("dispersionFunction<-")
exportMethods("dispersions<-")
exportMethods("normalizationFactors<-")
exportMethods("priorInfo<-")
exportMethods("sizeFactors<-")
exportMethods(counts)
exportMethods(design)
exportMethods(dispersionFunction)
exportMethods(dispersions)
exportMethods(estimateDispersions)
exportMethods(estimateSizeFactors)
exportMethods(normalizationFactors)
exportMethods(plotDispEsts)
exportMethods(plotMA)
exportMethods(plotPCA)
exportMethods(priorInfo)
exportMethods(show)
exportMethods(sizeFactors)
exportMethods(summary)
import(Biobase)
import(BiocGenerics)
import(BiocParallel)
import(GenomicRanges)
import(IRanges)
import(Rcpp)
import(S4Vectors)
import(SummarizedExperiment)
import(methods)
importFrom(MatrixGenerics,colMeans)
importFrom(MatrixGenerics,colSums)
importFrom(MatrixGenerics,rowMeans)
importFrom(MatrixGenerics,rowSums)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,hist)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(locfit,locfit)
importFrom(matrixStats,rowVars)
importFrom(stats,Gamma)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,coefficients)
importFrom(stats,df)
importFrom(stats,dnbinom)
importFrom(stats,dnorm)
importFrom(stats,formula)
importFrom(stats,glm)
importFrom(stats,loess)
importFrom(stats,lowess)
importFrom(stats,model.matrix)
importFrom(stats,optim)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,qf)
importFrom(stats,qnorm)
importFrom(stats,rchisq)
importFrom(stats,relevel)
importFrom(stats,rnbinom)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,splinefun)
importFrom(stats,terms)
importFrom(stats,terms.formula)
importFrom(stats4,summary)
importFrom(utils,askYesNo)
importFrom(utils,menu)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,read.table)
useDynLib(DESeq2)
