context("QR")
test_that("not using QR works as expected", {
  set.seed(1)
  dds <- makeExampleDESeqDataSet(n=100,betaSD=1)
  dds <- DESeq(dds, quiet=TRUE)
  ddsNoQR <- nbinomWaldTest(dds, useQR=FALSE)
  res <- results(dds)
  resNoQR <- results(ddsNoQR)
  expect_equal(res$log2FoldChange, resNoQR$log2FoldChange, tolerance=1e-6)
})
